SUBROUTINE USR_RATES( ijk, RATES )

      USE param
      USE param1
      USE parallel
      USE fldvar
      USE rxns
      USE energy
      USE geometry
      USE run
      USE indices
      USE physprop
      USE constant
      USE funits
      USE compar
      USE sendrecv
      USE toleranc
      USE functions
      USE fun_avg
      USE usr

      IMPLICIT NONE

      INTEGER, INTENT(IN) :: ijk
      DOUBLE PRECISION, DIMENSION(no_of_rxns), INTENT(OUT) :: RATES

      INCLUDE 'species.inc'
      INCLUDE 'usrnlst.inc'

      !!!! RATE SUBROUTINES !!!!
      call C3M_USR_RATES1( ijk, RATES )

      return
end

SUBROUTINE C3M_USR_RATES1( ijk, RATES )

      USE param
      USE param1
      USE parallel
      USE fldvar
      USE rxns
      USE energy
      USE geometry
      USE run
      USE indices
      USE physprop
      USE constant
      USE funits
      USE compar
      USE sendrecv
      USE toleranc
      USE functions
      USE fun_avg
      USE usr
! Reaction global variables:
!`````````````````````````````````````````````````````````````````````//
! Number of reactions.
      use rxns, only: NO_OF_RXNS
! Size of Reaction rates arrays.
      use rxns, only: nRR
! Array for storing reaction rates for output
      use rxns, only: ReactionRates
! Reaction names for debugging.
      use parse, only: RXN_NAME
      IMPLICIT NONE

      INTEGER, INTENT(IN) :: ijk
      DOUBLE PRECISION, DIMENSION(no_of_rxns), INTENT(OUT) :: RATES
      INTEGER :: L
      INCLUDE 'species.inc'
      INCLUDE 'usrnlst.inc'

!HMFUCracking {
!    chem_eq = "C6H6o3 --> 3*CO + 1.5*C2H4"
!}
!HAACracking {
!    chem_eq = "CH2OHCHO --> 2*CO + 2*H2"
!}
!GlyoxalCracking {
!    chem_eq = "CHOCHO --> 2*CO + H2"
!}
!AcetoneCracking {
!    chem_eq = "C2H5CHO --> 0.5*CO2 + 0.5*H2 + 1.25*C2H4"
!}
!PhenolCracking {
!    chem_eq = "C6H5OH --> 0.5*CO2 + 1.5*C2H4 + 2.5*soot"
!}
      DOUBLE PRECISION ::  k_cracking,E_cracking,T_gas, RT, KERT
      T_gas = min(2500.d0,T_g(ijk))
      k_cracking =   4.28d6 !/s
      E_cracking = 108.00d3 !j/mol
      RT         = 8.3145*T_gas ! j/mol/k * K = j/mol
      KERT       = k_cracking*exp(-E_cracking/RT) !/s


      RATES(HMFUCracking)    = ROP_g(IJK)*X_g(IJK,C6H6O3)/MW_G(C6H6O3)*KERT
      RATES(HAACracking)     = ROP_g(IJK)*X_g(IJK,CH2OHCHO)/MW_G(CH2OHCHO)*KERT
      RATES(GlyoxalCracking) = ROP_g(IJK)*X_g(IJK,CHOCHO)/MW_G(CHOCHO)*KERT
      RATES(AcetoneCracking) = ROP_g(IJK)*X_g(IJK,C2H5CHO)/MW_G(C2H5CHO)*KERT
      RATES(PhenolCracking)  = ROP_g(IJK)*X_g(IJK,C6H5OH)/MW_G(C6H5OH)*KERT


      DO L=1, NO_OF_RXNS
         ReactionRates(IJK,l) = RATES(L)         
      ENDDO  


      return
end
