!vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv!
!                                                                      !
!  Module name: USR_RATES                                              !
!  Author: J.Musser                                   Date: 10-Oct-12  !
!                                                                      !
!  Purpose: Hook for user defined reaction rates.                      !
!                                                                      !
!``````````````````````````````````````````````````````````````````````!
!
!                                                                      !
!^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^!
      SUBROUTINE USR_RATES_DES(NP, pM, IJK, DES_RATES)

! Number of DES chemical reactions
      use des_rxns

! Gas phase global variables:
!`````````````````````````````````````````````````````````````````````//
      use physprop, only: MW_g, X_s0
      use physprop, only: MW_MIX_g, ro_xs0, d_p0
      use fldvar, only: T_g, P_g, X_g, RO_g
! Gas phase global variables:
!`````````````````````````````````````````````````````````````````````//
! Particle temperutre
      use des_thermo, only: DES_T_s
      use discretelement, only:CGDEM , des_usr_var, RO_SOL, DES_RADIUS, PMASS
      use discretelement, only: DES_CGP_RPR, DES_CGP_STW
! Solids species molecular weights
      use physprop, only: MW_S, NMAX_S
! Particle species mass fraction
      use des_rxns, only: DES_X_s

! Reaction global variables:
!`````````````````````````````````````````````````````````````````````//
! Number of reactions.
      use rxns, only: NO_OF_RXNS
! Array for storing reaction rates for output
      use rxns, only: ReactionRates
! Reaction names for debugging.
      use parse, only: RXN_NAME

! Global parameters:
!`````````````````````````````````````````````````````````````````````//
! Gas constant (cal/mol.K)
      use constant, only: RGAS => GAS_CONST_cal, Pi
! Double percision aliases:
      use param1, only: ZERO
      use param1, only: SMALL_NUMBER
      use param1, only: HALF
      use param1, only: ONE
      use ic, only: IC_PIC_CONST_STATWT
      use run, only: DT,time

! User-specified constant
      use constant, only: C
! Function to calculate Cp over gas constant R
      use read_thermochemical, only: calc_CpoR
      use calc_gamma_des_mod, only: calc_gamma_des

      IMPLICIT NONE

      INTEGER, INTENT(IN) :: NP  ! Global index of particle
      INTEGER, INTENT(IN) :: pM  ! Solid phase index of particle NP
      INTEGER, INTENT(IN) :: IJK ! Fluid cell index containing NP

! Calculated reaction rates. (reacted moles per sec)
      DOUBLE PRECISION, INTENT(OUT) :: DES_RATES(NO_OF_DES_RXNS)

! Included parameters and functions:
!`````````````````````````````````````````````````````````````````````//
      include 'species.inc'
      include 'usrnlst.inc'


! Reaction specific variables:
!`````````````````````````````````````````````````````````````````````//
! Index used for looping over all reactions.
      INTEGER L

! Bounded Phase temperatues (K)
      DOUBLE PRECISION, parameter :: MAX_TEMP = 2.5d3
      DOUBLE PRECISION, parameter :: residuleMass = ZERO 
      DOUBLE PRECISION :: Tp, RT, mp
      DOUBLE PRECISION :: ReactantMass

      ! skip sands
      if(pM==2) then
               DES_RATES(:) = 0.d0
         return
      endif

      !set to 773K
      !DES_T_S(NP) = min(773.d0, 298.d0 + (773-298)/4.d0*time)

      Tp = min(MAX_TEMP, DES_T_S(NP))
      MP = pmass(NP)
      
!---------------------------------------------------------------------//
!---------------------------------------------------------------------//
! Pyrolysis:  Volatiles -->
!---------------------------------------------------------------------//
       RT = 1.987*Tp
       !-------------- drying   -----------
      ReactantMass  = DES_X_s(NP,MOISTURE)*pmass(NP)
      IF(ReactantMass .GT. residuleMass) DES_RATES(drying) = Tp*exp(-8000/RT)*ReactantMass/MW_s(1,MOISTURE)

!Cellulose
       ReactantMass = Mp*DES_X_s(NP,CELL_C6H10O5)
       IF(ReactantMass .GT. residuleMass) THEN
         DES_RATES(Reaction_1) = 1.5d14*exp(-47000./(RT))* ReactantMass/MW_s(pM,CELL_C6H10O5)
         DES_RATES(Reaction_4) = 9.d7*exp(-31000./(RT))* ReactantMass/MW_s(pM,CELL_C6H10O5)
       ENDIF

       ReactantMass = Mp*DES_X_s(NP,CELLA_C6H10O5)
       IF(ReactantMass .GT. residuleMass) THEN
         DES_RATES(Reaction_2) = 2.5d6*exp(-19100./(RT))* ReactantMass/MW_s(pM,CELLA_C6H10O5)
         DES_RATES(Reaction_3) = 3.3*Tp*exp(-10000./(RT))* ReactantMass/MW_s(pM,CELLA_C6H10O5)
       ENDIF

!Hemicellulose
       ReactantMass = Mp*DES_X_s(NP,GMSW_C5H8O4)
       IF(ReactantMass .GT. residuleMass)  &
         DES_RATES(Reaction_5) = 1.d10*exp(-31000./(RT))* ReactantMass/MW_s(pM,GMSW_C5H8O4)

       ReactantMass = Mp*DES_X_s(NP,XYHW_C5H8O4)
       IF(ReactantMass .GT. residuleMass) &
         DES_RATES(Reaction_6) = 1.25d11*exp(-31400./(RT))* ReactantMass/MW_s(pM,XYHW_C5H8O4)

       ReactantMass = Mp*DES_X_s(NP,XYGR_C5H8O4)
       IF(ReactantMass .GT. residuleMass)  &
         DES_RATES(Reaction_7) = 1.25d11*exp(-30000./(RT))* ReactantMass/MW_s(pM,XYGR_C5H8O4)

       ReactantMass = Mp*DES_X_s(NP,HCE1_C5H8O4)
       IF(ReactantMass .GT. residuleMass) THEN
         DES_RATES(Reaction_8) = 16.*Tp*exp(-12900./(RT))* ReactantMass/MW_s(pM,HCE1_C5H8O4)
         DES_RATES(Reaction_9) = 3.d-3*Tp*exp(-3600./(RT))* ReactantMass/MW_s(pM,HCE1_C5H8O4)
       ENDIF

       ReactantMass = Mp*DES_X_s(NP,HCE2_C5H8O4)
       IF(ReactantMass .GT. residuleMass)  &
         DES_RATES(Reaction_10) = 7.d9*exp(-30500./(RT))* ReactantMass/MW_s(pM,HCE2_C5H8O4)

!Lignins
       ReactantMass = Mp*DES_X_s(NP,LIGH_C22H28O9)
       IF(ReactantMass .GT. residuleMass)  &
         DES_RATES(Reaction_11) = 6.7d12*exp(-37500./(RT))* ReactantMass/MW_s(pM,LIGH_C22H28O9)

       ReactantMass = Mp*DES_X_s(NP,LIGO_C20H22O10)
       IF(ReactantMass .GT. residuleMass)  &
         DES_RATES(Reaction_12) = 3.3d8*exp(-22500./(RT))* ReactantMass/MW_s(pM,LIGO_C20H22O10)

       ReactantMass = Mp*DES_X_s(NP,LIGC_C15H14O4)
       IF(ReactantMass .GT. residuleMass)  &
         DES_RATES(Reaction_13) = 1.d11*exp(-37200./(RT))* ReactantMass/MW_s(pM,LIGC_C15H14O4)

       ReactantMass = Mp*DES_X_s(NP,LIGCC_C15H14O4)
       IF(ReactantMass .GT. residuleMass)  &
         DES_RATES(Reaction_14) = 1.d4*exp(-24800./(RT))* ReactantMass/MW_s(pM,LIGCC_C15H14O4)

       ReactantMass = Mp*DES_X_s(NP,LIGOH_C19H22O8)
       IF(ReactantMass .GT. residuleMass)  &
         DES_RATES(Reaction_15) = 1.5d8*exp(-30000./(RT))* ReactantMass/MW_s(pM,LIGOH_C19H22O8)

       ReactantMass = Mp*DES_X_s(NP,LIG_C11H12O4)
       IF(ReactantMass .GT. residuleMass) THEN
         DES_RATES(Reaction_16) = 4.*Tp*exp(-12000./(RT))* ReactantMass/MW_s(pM,LIG_C11H12O4)
         DES_RATES(Reaction_17) = 8.3d-2*Tp*exp(-8000./(RT))* ReactantMass/MW_s(pM,LIG_C11H12O4)
         DES_RATES(Reaction_18) = 1.5d9*exp(-31500./(RT))* ReactantMass/MW_s(pM,LIG_C11H12O4)
       ENDIF

!Extractives
       ReactantMass = Mp*DES_X_s(NP,TGL_C57H10O7)
       IF(ReactantMass .GT. residuleMass) &
         DES_RATES(Reaction_19) = 7.d12*exp(-45700./(RT))* ReactantMass/MW_s(pM,TGL_C57H10O7)

       ReactantMass = Mp*DES_X_s(NP,TANN_C15H12O7)
       IF(ReactantMass .GT. residuleMass)  &
         DES_RATES(Reaction_20) = 2.d1*exp(-10000./(RT))* ReactantMass/MW_s(pM,TANN_C15H12O7)

       ReactantMass = Mp*DES_X_s(NP,ITANN_C8H4O4)
       IF(ReactantMass .GT. residuleMass)  &
         DES_RATES(Reaction_21) = 1.d3*exp(-25000./(RT))* ReactantMass/MW_s(pM,ITANN_C8H4O4)

!Metaplastic
       ReactantMass = Mp*DES_X_s(NP,G_CO2)
       IF(ReactantMass .GT. residuleMass)  &
         DES_RATES(Reaction_22) = 1.d6*exp(-24500./(RT))* ReactantMass/MW_s(pM,G_CO2)

       ReactantMass = Mp*DES_X_s(NP,G_CO)
       IF(ReactantMass .GT. residuleMass)  &
         DES_RATES(Reaction_23) = 5.d12*exp(-52500./(RT))* ReactantMass/MW_s(pM,G_CO)

       ReactantMass = Mp*DES_X_s(NP,G_CH3OH)
       IF(ReactantMass .GT. residuleMass)  &
         DES_RATES(Reaction_24) = 2.d12*exp(-50000./(RT))* ReactantMass/MW_s(pM,G_CH3OH)

       ReactantMass = Mp*DES_X_s(NP,G_COH2_L)
       IF(ReactantMass .GT. residuleMass)  &
         DES_RATES(Reaction_25) = 6.d10*exp(-50000./(RT))* ReactantMass/MW_s(pM,G_COH2_L)

       ReactantMass = Mp*DES_X_s(NP,G_C2H6)
       IF(ReactantMass .GT. residuleMass)  &
         DES_RATES(Reaction_26) = 1.d11*exp(-52000./(RT))* ReactantMass/MW_s(pM,G_C2H6)

       ReactantMass = Mp*DES_X_s(NP,G_CH4)
       IF(ReactantMass .GT. residuleMass)  &
         DES_RATES(Reaction_27) = 1.d11*exp(-53000./(RT))* ReactantMass/MW_s(pM,G_CH4)

       ReactantMass = Mp*DES_X_s(NP,G_C2H4)
       IF(ReactantMass .GT. residuleMass)  &
         DES_RATES(Reaction_28) = 1.d11*exp(-54000./(RT))* ReactantMass/MW_s(pM,G_C2H4)

       ReactantMass = Mp*DES_X_s(NP,G_C6H5OH)
       IF(ReactantMass .GT. residuleMass)  &
         DES_RATES(Reaction_29) = 1.5d12*exp(-55000./(RT))* ReactantMass/MW_s(pM,G_C6H5OH)

       ReactantMass = Mp*DES_X_s(NP,G_COH2_S)
       IF(ReactantMass .GT. residuleMass)  &
         DES_RATES(Reaction_30) = 1.d9*exp(-59000./(RT))* ReactantMass/MW_s(pM,G_COH2_S)

       ReactantMass = Mp*DES_X_s(NP,G_H2)
       IF(ReactantMass .GT. residuleMass)  &
         DES_RATES(Reaction_31) = 1.d8*exp(-70000./(RT))* ReactantMass/MW_s(pM,G_H2)

       RETURN

       END SUBROUTINE USR_RATES_DES
